'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var reactRouterDom = require('react-router-dom');
var hoistNonReactStatics = _interopDefault(require('hoist-non-react-statics'));

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

/**
 * renderRoutes
 * help building <Route> list from a route config
 *
 * @param {Object} rootConfig root routes config
 * @param {number} level level
 * @return {React.Component}
 */

var renderRoutes = function renderRoutes(rootConfig) {
  var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

  if (rootConfig.component) {
    return React.createElement(rootConfig.Route, _objectSpread({}, rootConfig, {
      key: level
    }));
  }

  if (rootConfig.subRoutes) {
    // indicates the route at this level do not need to an explicit route
    // which is the kind of route do not have a corresponding content component to be rendered
    // 如果有 path 但没有 component 的，说明这层是 route，但不需要显式展现
    // all the route need to do is redirecting when matched
    // 对于不需要显式显示的 Route 访问的时候直接 Redirect
    // as for redirecting, <Route> and <Redirect> need to be wrap by <Switch>
    // 所以也一定需要 Switch 包裹
    // generate a component following the above requirement
    // 根据以上逻辑生成 component
    if (rootConfig.path && !rootConfig.component) {
      rootConfig.component = function (props) {
        return renderRoutes({
          subRoutes: props.subRoutes,
          isRedirect: true,
          isSwitch: true
        });
      };

      return renderRoutes(rootConfig);
    }

    var children = rootConfig.subRoutes.map(function (routeConfig, index) {
      return renderRoutes(routeConfig, "".concat(level, "_").concat(index));
    });

    if (rootConfig.isRedirect) {
      children.push(React.createElement(reactRouterDom.Redirect, {
        to: rootConfig.subRoutes[0].path,
        key: "".concat(level, "_").concat(children.length)
      }));
    }

    return rootConfig.isSwitch ? React.createElement(reactRouterDom.Switch, {
      key: level
    }, children) : children;
  }
};

var getDisplayName = (function (Component) {
  return Component.displayName || Component.name || 'Component';
});

/**
 * The HOC which return new Route with the capability of passing props on <Route> to Component
 * 能够生成将 <Route> 上的（部分） props 传给 <Route> 对应的 Component 的 Route 的 HOC
 *
 * @param {Array} componentPropKeyList keyname list of props need to be passed to component
 * @param {?React.Component} ParentRoute wrappedRoute
 * @return {React.Component} result component
 */

var withPropsPassing = (function (componentPropKeyList) {
  var ParentRoute = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : reactRouterDom.Route;

  var withPropsPassingRoute = function withPropsPassingRoute(_ref) {
    var Component = _ref.component,
        restProps = _objectWithoutProperties(_ref, ["component"]);

    var componentProps = function () {
      var result = {};
      componentPropKeyList.forEach(function (key) {
        if (key in restProps) {
          result[key] = restProps[key];
        }
      });
      return result;
    }(); // if no Component specified (render or children may included), pass restProps directly
    // 没有 component 时，直接按照 restProps 的设定，比如 render 来处理 Route


    if (!Component) {
      return React.createElement(ParentRoute, _objectSpread({}, restProps));
    }

    return React.createElement(ParentRoute, _objectSpread({}, restProps, {
      render: function render(childProps) {
        return React.createElement(Component, _objectSpread({}, childProps, componentProps));
      }
    }));
  };

  withPropsPassingRoute.displayName = "WithPropsPassing(".concat(getDisplayName(ParentRoute), ")");
  withPropsPassingRoute.WrappedComponent = ParentRoute;
  return hoistNonReactStatics(withPropsPassingRoute, ParentRoute);
});

// 构建出可以将 <Route> 上的 subRoutes 直接向 Component 传递的 <BRoute>，方便通过统一配置生成一系列 <Route>

var BRoute = withPropsPassing(['subRoutes']);

exports.renderRoutes = renderRoutes;
exports.BRoute = BRoute;
exports.withPropsPassing = withPropsPassing;
